<?php

/**
 * @file
 * ALB Authentication module admin settings.
 */

/**
 * The UI controller for claim mappings.
 */
class AlbAuthUIController extends EntityDefaultUIController {

  /**
   * {@inheritdoc}
   */
  protected function overviewTableHeaders($conditions, $rows, $additional_header = array()) {
    $additional_header[] = t('Enabled');
    if ($this->entityType == 'alb_auth_claim_map') {
      $additional_header[] = t('ALB instance');
    }
    return parent::overviewTableHeaders($conditions, $rows, $additional_header);
  }

  /**
   * {@inheritdoc}
   */
  protected function overviewTableRow($conditions, $id, $entity, $additional_cols = array()) {
    $additional_cols[] = !empty($entity->enabled) ? t('Yes') : t('No');
    if ($this->entityType == 'alb_auth_claim_map') {
      $additional_cols[] = $entity->alb_instance;
    }
    return parent::overviewTableRow($conditions, $id, $entity, $additional_cols);
  }

}

/**
 * ALB Authentication settings form.
 */
function alb_auth_settings_form($form, &$form_state) {
  $claims = alb_auth_get_claims_from_request();

  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t("Enable ALB authentication. Check that it's working first."),
    '#default_value' => alb_auth_enabled(),
    '#disabled' => !$claims->hasData(),
  );

  $form['verbose'] = array(
    '#type' => 'checkbox',
    '#title' => t('Verbose logging'),
    '#description' => t("Enable verbose logging of ALB auth events."),
    '#default_value' => alb_auth_verbose_log_enabled(),
  );

  $form['actions']['#type'] = 'actions';
  $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));

  $form['claims'] = alb_auth_admin_claims_table();

  return $form;
}

/**
 * ALB Authentication settings form submission callback.
 */
function alb_auth_settings_form_submit($form, &$form_state) {
  variable_set('alb_auth_settings', $form_state['values']['enabled']);
  variable_set('alb_auth_verbose_log', $form_state['values']['verbose']);

  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * Returns the render array of a table of ALB claims from current request.
 */
function alb_auth_admin_claims_table() {
  $claims = alb_auth_get_claims_from_request()->toArray();

  $rows = [];
  if ($claims) {
    foreach ($claims as $name => $value) {
      $rows[] = [
        $name,
        $value,
      ];
    }
  }

  $claims_table = [
    '#weight' => 100,
    '#type' => 'item',
    '#title' => t('Active claims'),
    '#description' => t('Claims available from the current request. The list if provided for reference.'),
    'list' => [
      '#theme' => 'table',
      '#header' => [t('Name'), t('Value')],
      '#empty' => t('No claims found - is everything set up correctly?'),
      '#rows' => $rows,
    ],
  ];

  return $claims_table;
}

/**
 * Claim mapping edit form.
 */
function alb_auth_claim_map_form($form, &$form_state, $claim_map, $op = 'edit', $entity_type = NULL) {
  if ($op == 'clone') {
    // Only label is provided for cloned entities.
    $claim_map->label .= ' ' . t('(cloned)');
    $claim_map->type = $entity_type . '_clone';
  }

  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t("Enable the mapping"),
    '#default_value' => !empty($claim_map->enabled),
  );

  $form['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => isset($claim_map->label) ? $claim_map->label : '',
  );

  // Machine-readable type name.
  $form['name'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($claim_map->name) ? $claim_map->name : '',
    '#machine_name' => array(
      // TODO: Fix me, this thing doesn't let me to edit the mapping!
      'exists' => 'alb_auth_claim_map_load',
      'source' => array('label'),
    ),
    '#disabled' => empty($claim_map->is_new),
    '#description' => t('A unique machine-readable name for this the claim map. It must only contain lowercase letters, numbers, and underscores.'),
  );

  $form['alb_instance'] = array(
    '#type' => 'select',
    '#title' => t('ALB instance'),
    '#default_value' => isset($claim_map->alb_instance) ? $claim_map->alb_instance : '',
    '#required' => TRUE,
    '#options' => alb_auth_admin_instance_options(),
  );

  // TODO: Add claim source field autocomplete.
  $form['claim_source'] = array(
    '#type' => 'textfield',
    '#title' => t('Claim source'),
    '#default_value' => isset($claim_map->claim_source) ? $claim_map->claim_source : '',
    '#description' => t('Name of the JWT claim to map from.'),
    '#required' => TRUE,
  );

  // TODO: The target should be a property/field selection dropdown.
  $form['mapping_target'] = array(
    '#type' => 'textfield',
    '#title' => t('Mapping target'),
    '#default_value' => isset($claim_map->mapping_target) ? $claim_map->mapping_target : '',
    '#description' => t('User entity field to map to.'),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save claim mapping'),
    '#weight' => 40,
  );

  $form['claims'] = alb_auth_admin_claims_table();

  return $form;
}

/**
 * Form API submit callback for the type form.
 */
function alb_auth_claim_map_form_submit(&$form, &$form_state) {
  $claim_map = entity_ui_form_submit_build_entity($form, $form_state);
  // Save and go back.
  $claim_map->save();
  $form_state['redirect'] = 'admin/config/people/alb_auth/mapping';
}

/**
 * Claim mapping edit form.
 */
function alb_auth_instance_form($form, &$form_state, $alb_instance, $op = 'edit', $entity_type = NULL) {
  if ($op == 'clone') {
    // Only label is provided for cloned entities.
    $alb_instance->label .= ' ' . t('(cloned)');
    $alb_instance->type = $entity_type . '_clone';
  }

  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t("Enable the instance"),
    '#default_value' => !empty($alb_instance->enabled),
  );

  $form['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($alb_instance->label) ? $alb_instance->label : '',
  );

  // Machine-readable type name.
  $form['name'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($alb_instance->name) ? $alb_instance->name : '',
    '#machine_name' => array(
      // TODO: Fix me, this thing doesn't let me to edit the mapping!
      'exists' => 'alb_auth_instance_load',
      'source' => array('label'),
    ),
    '#disabled' => empty($alb_instance->is_new),
    '#description' => t('An unique machine-readable name for this ALB instance. It must only contain lowercase letters, numbers, and underscores.'),
  );

  $form['aws_region'] = array(
    '#type' => 'textfield',
    '#title' => t('AWS region'),
    '#description' => t('AWS region of the Application Load Balancer'),
    '#required' => TRUE,
    '#default_value' => isset($alb_instance->aws_region) ? $alb_instance->aws_region : '',
  );
  $form['aws_arn'] = array(
    '#type' => 'textfield',
    '#title' => t('AWS ARN'),
    '#description' => t('Unique AWS resource identifier'),
    '#required' => TRUE,
    '#default_value' => isset($alb_instance->aws_arn) ? $alb_instance->aws_arn : '',
  );
  $form['cookie_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Cookie name'),
    '#description' => t('ALB authentication cookie name. Note that the ALB seems to add a prefix of -0 to what you define.'),
    '#required' => TRUE,
    '#default_value' => isset($alb_instance->cookie_name) ? $alb_instance->cookie_name : '',
  );
  $form['provision'] = array(
    '#type' => 'checkbox',
    '#title' => t('Provisioning'),
    '#description' => t("Create users automatically if they don't already exist in Drupal."),
    '#default_value' => isset($alb_instance->provision) ? $alb_instance->provision : '',
  );
  $form['oidc_data_header'] = array(
    '#type' => 'textfield',
    '#title' => t('OIDC data header name'),
    '#description' => t("The name of the header used for ALB auth claims token. Do not change unless you know what you're doing."),
    '#required' => TRUE,
    '#default_value' => isset($alb_instance->oidc_data_header) ? $alb_instance->oidc_data_header : 'x-amzn-oidc-data',
  );
  $form['logout_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Logout settings'),
    '#collapsible' => FALSE,
  );
  $form['logout_fieldset']['logout'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t("Enable logout for this ALB instance"),
    '#default_value' => !empty($alb_instance->logout),
  );
  $form['logout_fieldset']['cognito_base_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Cognito base URL'),
    '#default_value' => isset($alb_instance->cognito_base_url) ? $alb_instance->cognito_base_url : '',
  );
  $form['logout_fieldset']['cognito_client_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Cognito client ID'),
    '#default_value' => isset($alb_instance->cognito_client_id) ? $alb_instance->cognito_client_id : '',
  );
  $form['logout_fieldset']['logout_redirect_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Logout redirect URL'),
    '#description' => t('A URL to redirect user after Cognito logout. Either an absolute URL, an internal Drupal path or <front>.'),
    '#default_value' => isset($alb_instance->logout_redirect_url) ? $alb_instance->logout_redirect_url : '',
  );
  $form['module'] = array(
    '#type' => 'hidden',
    '#default_value' => 'alb_auth',
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save ALB instance'),
    '#weight' => 40,
  );

  // TODO: Only return claims for a given request header.
  // TODO: AJAX update of claims on header name update.
  $form['claims'] = alb_auth_admin_claims_table();

  return $form;
}

/**
 * Form API submit callback for the type form.
 */
function alb_auth_instance_form_submit(&$form, &$form_state) {
  $alb_instance = entity_ui_form_submit_build_entity($form, $form_state);
  // Add default mappings.
  if (!empty($alb_instance->is_new)) {
    $default_maps = alb_auth_default_alb_auth_claim_map();
    foreach ($default_maps as $alb_map) {
      $alb_map->alb_instance = $alb_instance->name;
      $alb_map->name .= '_' . $alb_instance->name;
      entity_save('alb_auth_claim_map', $alb_map);
    }
  }
  // Save and go back.
  $alb_instance->save();
  $form_state['redirect'] = 'admin/config/people/alb_auth/instance';
}

/**
 * Returns a list of ALB instances for the select form element.
 */
function alb_auth_admin_instance_options() {
  $options = array();
  foreach (entity_load('alb_auth_instance') as $instance) {
    $options[$instance->name] = $instance->label;
  }
  return $options;
}
