<?php

/**
 * @file
 * Hooks provided by the ALB Auth module.
 */

/**
 * An alter hook for user properties retrieved from ALB claims.
 *
 * @param array $properties
 *   An array of user properties.
 * @param AlbAuthClaimCollectionInterface $claims
 *   ALB claims collection.
 */
function hook_alb_auth_user_properties_alter(array &$properties, AlbAuthClaimCollectionInterface $claims) {
  // An example setting the user blogger role.
  if ($claims->getClaimValue('user_is_blogger', FALSE)
    && $role = user_role_load_by_name('blogger')) {
    $rid = $role->rid;
    $properties['roles'][$rid] = $rid;
  }
}

/**
 * Alter ID of user loaded initially.
 *
 * @param int|null $uid
 *   User ID or NULL.
 * @param AlbAuthClaimCollectionInterface $claims
 *   ALB claims collection.
 *
 * @see alb_auth_initial_user_load()
 */
function hook_alb_auth_initial_user_alter(&$uid, AlbAuthClaimCollectionInterface $claims) {
  if (!$uid && $ext_id = $claims->getClaimValue('external_id')) {
    // Try retrieving user ID by matching the external_id claim.
    // NOTE: There's no such claim (unless you added it in your Cognito config),
    // it's only used as an example.
    $uid = db_select('my_module_ext_map', 't')
      ->fields('t', array('uid'))
      ->condition('t.ext_id', $ext_id)
      ->execute()
      ->fetchField();
  }
}

/**
 * React on ALB authorization.
 *
 * @param object $account
 *   Object of the user.
 *
 * @throws \Exception
 *
 * @see alb_auth_force_relogin()
 */
function hook_alb_auth_user_relogin() {
  global $user;
  if (!$user->status) {
    user_save($user, array('status' => 1));
  }
}

/**
 * React on saving Drupal to Cognito user ID mapping.
 *
 * @param int $uid
 *   An ID of user.
 * @param AlbAuthClaimCollectionInterface $claims
 *   Claims collection.
 *
 * @see alb_auth_save_authmap()
 */
function hook_alb_auth_authmap_save($uid, AlbAuthClaimCollectionInterface $claims) {
  my_module_store_user_map($uid, $claims->toArray());
}

/**
 * React on Cognito claims change.
 *
 * The purpose of this hook is updating custom user mapping.
 *
 * @param object $account
 *   Object of the user.
 * @param AlbAuthClaimCollectionInterface $claims
 *   Claims collection.
 *
 * @see alb_auth_update_user_with_claims()
 */
function hook_alb_auth_authmap_update($account, AlbAuthClaimCollectionInterface $claims) {
  my_module_update_user_map($account->uid, $claims->toArray());
}

/**
 * Forbid ALB authentication.
 *
 * @see alb_auth_allowed()
 */
function hook_alb_auth_allowed() {
  if (module_exists('simplesamlphp_auth')) {
    return FALSE;
  }
}
