<?php

/**
 * @file
 * The collection of ALB claims mappings functions.
 */

/**
 * Creates a new user account from ALB claims.
 *
 * @param \AlbAuthClaimCollectionInterface $claims
 *   The claims collection.
 *
 * @return \stdClass|null
 *   The user account object or NULL.
 */
function alb_auth_create_user_from_claims(AlbAuthClaimCollectionInterface $claims, $alb_instance) {
  $values = alb_auth_map_claims_collection_to_fields($claims, $alb_instance->name);
  $values += array(
    'roles' => array(DRUPAL_AUTHENTICATED_RID => DRUPAL_AUTHENTICATED_RID),
    'pass' => user_password(),
    'status' => 1,
    'timezone' => variable_get('date_default_timezone', ''),
    'access' => REQUEST_TIME,
    'is_new' => TRUE,
  );
  if (isset($values['mail'])) {
    $values['init'] = $values['mail'];
  }
  if (empty($values['name']) || empty($values['mail'])) {
    watchdog('alb_auth', 'Failed creating the user from claims. Either name or mail field is empty.', array(), WATCHDOG_ERROR);
    return NULL;
  }
  $values['name'] = alb_auth_unique_username($values['name']);
  if ($account = user_save(NULL, $values)) {
    // Save users claim version.
    alb_auth_update_users_claim_version($account, $alb_instance->name, $values);
  }
  return $account;
}

/**
 * Updates user account with values from ALB claims.
 *
 * @param mixed $account
 *   The user account object.
 * @param \AlbAuthClaimCollectionInterface $claims
 *   The claims collection.
 * @param object $alb_instance
 *   ALB instance entity.
 *
 * @return object
 *   User object.
 */
function alb_auth_update_user_with_claims($account, AlbAuthClaimCollectionInterface $claims, $alb_instance) {
  $edit = array();
  // Do not update user if claims configuration and claims data didn't change.
  if (alb_auth_should_update_user($account, $claims, $alb_instance->name)) {
    $user_properties = entity_get_property_info('user');
    $values = alb_auth_map_claims_collection_to_fields($claims, $alb_instance->name);
    alb_auth_update_users_claim_version($account, $alb_instance->name, $values);
    foreach ($values as $property => $value) {
      if (isset($user_properties['properties'][$property])) {
        if ($property === 'name') {
          // Skip updating user name.
          continue;
        }
        $edit[$property] = $value;
      }
      else {
        watchdog('alb_auth', 'Target property @property does not exist.', array('@property' => $property), WATCHDOG_WARNING);
      }
    }

    module_invoke_all('alb_auth_authmap_update', $account, $claims);
  }

  if (!empty($edit)) {
    user_save($account, $edit);
  }
  return $account;
}

/**
 * Loads not yet mapped user by values claims.
 *
 * Normally this method looks up user by email, however, it's possible to alter
 * the UID with a hook implementation.
 *
 * @param AlbAuthClaimCollectionInterface $claims
 *   Claims collection.
 * @param object $alb_instance
 *   ALB instance entity.
 *
 * @return object|null
 *   The user account entity or NULL.
 *
 * @see hook_alb_auth_initial_user_alter()
 */
function alb_auth_initial_user_load(AlbAuthClaimCollectionInterface $claims, $alb_instance) {
  if ($claim_mapping = alb_auth_claim_map_load_by_target('mail', $alb_instance->name)) {
    if (!($email = $claims->getClaimValue($claim_mapping->claim_source))) {
      $error_message = 'Could not determine email from ALB claims.';
    }
  }
  else {
    $error_message = 'Could not find claim mapping for the mail field.';
  }

  $uid = NULL;
  if (!empty($email) && $user = user_load_by_mail($email)) {
    // Try looking up user by email.
    $uid = $user->uid;
  }

  // Give other modules a chance to change our opinion.
  drupal_alter('alb_auth_initial_user', $uid, $claims);

  if (empty($uid)) {
    if (isset($error_message)) {
      // Log error message if UID was not found and none of the modules provided
      // it.
      watchdog('alb_auth', $error_message, array(), WATCHDOG_ERROR);
    }

    return NULL;
  }

  return user_load($uid);
}

/**
 * Returns all enabled claims mappings.
 *
 * @param string $alb_instance
 *   The ALB instance name.
 *
 * @return array
 *   And array of enabled claim mapping entities.
 */
function alb_auth_claim_maps_load_enabled($alb_instance) {
  return entity_load('alb_auth_claim_map', FALSE, array(
    'enabled' => 1,
    'alb_instance' => $alb_instance,
  ));
}

/**
 * Returns the claim mapping for a given target property.
 *
 * @param string $target
 *   Target user property.
 * @param string $alb_instance
 *   The ALB instance name.
 *
 * @return object|null
 *   The claim mapping entity or NULL.
 */
function alb_auth_claim_map_load_by_target($target, $alb_instance) {
  $claim_mappings = entity_load('alb_auth_claim_map', FALSE, array(
    'mapping_target' => $target,
    'alb_instance' => $alb_instance,
  ));
  return reset($claim_mappings);
}

/**
 * Processes claims collection and returns user properties according to mapping.
 *
 * @param AlbAuthClaimCollectionInterface $claims
 *   The claims collection.
 * @param string $alb_instance
 *   The ALB instance name.
 *
 * @return array
 *   An array of user claims.
 */
function alb_auth_map_claims_collection_to_fields(AlbAuthClaimCollectionInterface $claims, $alb_instance) {
  $values = [
    'alb_auth_claims_hash' => $claims->hash(),
    'alb_auth_mapping_version' => alb_auth_claims_mapping_version($alb_instance),
  ];
  foreach (alb_auth_claim_maps_load_enabled($alb_instance) as $claim_mapping) {
    if ($claim = $claims->getClaimByName($claim_mapping->claim_source)) {
      $values[$claim_mapping->mapping_target] = $claim->getValue();
    }
  }
  drupal_alter('alb_auth_user_properties', $values, $claims);
  return $values;
}

/**
 * Update claim version.
 *
 * @param object $account
 *   The user object.
 * @param string $alb_instance
 *   The ALB instance name.
 * @param array $properties
 *   The user properties according to mapping.
 *
 * @throws \Exception
 */
function alb_auth_update_users_claim_version($account, $alb_instance, array $properties) {
  // User edited data.
  $record = array(
    'claims_hash' => $properties['alb_auth_claims_hash'],
    'mapping_version' => $properties['alb_auth_mapping_version'],
  );

  db_merge('alb_auth_users_claims')
    ->key(array('uid' => $account->uid, 'alb_instance' => $alb_instance))
    ->fields($record)
    ->execute();
}

/**
 * Checks whether user account should be updated with claims.
 *
 * @param object $account
 *   User account entity.
 * @param AlbAuthClaimCollectionInterface $claims
 *   The claims collection.
 * @param string $alb_instance
 *   The ALB instance name.
 *
 * @return bool
 *   Whether the user account should be updated with claims.
 */
function alb_auth_should_update_user($account, AlbAuthClaimCollectionInterface $claims, $alb_instance) {
  // Update user if one of the following conditions is met:
  // - Claims mapping configuration was updated since last claims data import.
  // - ALB claims were updated.
  $user_claims_properties = alb_auth_get_user_claims_properties($account, $alb_instance);

  $user_claims_hash = $user_claims_properties->claims_hash;
  $token_claims_hash = $claims->hash();
  $user_mapping_version = $user_claims_properties->mapping_version;
  $token_mapping_version = alb_auth_claims_mapping_version($user_claims_properties->alb_instance);

  $changed = $user_mapping_version != $token_mapping_version
    || $user_claims_hash != $token_claims_hash;
  if ($changed) {
    alb_auth_verbose_log('User @uid claims have changed, old hash: @old_hash, new hash: @new_hash, old mapping: @old_map, new mapping: @new_map', [
      '@old_hash' => $user_claims_hash,
      '@new_hash' => $token_claims_hash,
      '@old_map' => $user_mapping_version,
      '@new_map' => $token_mapping_version,
    ]);
  }
  return $changed;
}

/**
 * Returns current claims mapping configuration version.
 *
 * @param string|null $alb_instance
 *   The ALB instance name.
 *
 * @return int|array
 *   The mapping configuration version if ALB instance is set or array of all
 *   versions.
 */
function alb_auth_claims_mapping_version($alb_instance = NULL) {
  $mapping_versions = variable_get('alb_auth_claims_mapping_version', array());
  if (!$alb_instance) {
    return $mapping_versions;
  }
  else {
    return isset($mapping_versions[$alb_instance]) ? $mapping_versions[$alb_instance] : 0;
  }
}

/**
 * Increments claims mapping configuration version.
 *
 * @param string $alb_instance
 *   The ALB instance name.
 */
function alb_auth_claims_mapping_version_increment($alb_instance) {
  $mapping_versions = alb_auth_claims_mapping_version();
  $mapping_versions[$alb_instance] = alb_auth_claims_mapping_version($alb_instance) + 1;
  variable_set('alb_auth_claims_mapping_version', $mapping_versions);
}

/**
 * Returns unique Drupal username based on ALB username claim.
 *
 * @param string $username
 *   ALB claims username.
 *
 * @return string
 *   The unique Drupal username.
 */
function alb_auth_unique_username($username) {
  $count = 0;
  while (TRUE) {
    $possible_username = $count > 0 ? $username . '_' . $count : $username;
    if (!user_load_by_name($possible_username)) {
      return $possible_username;
    }

    $count++;
  }
}

/**
 * Get saved claim properties of the user.
 *
 * @param object|null $account
 *   The user object.
 * @param string $alb_instance
 *   The current ALB instance name.
 *
 * @return object
 *   The claim object.
 */
function alb_auth_get_user_claims_properties($account, $alb_instance) {
  if (!$account) {
    global $user;
    $account = $user;
  }
  return db_select('alb_auth_users_claims')
    ->fields('alb_auth_users_claims')
    ->condition('uid', $account->uid)
    ->condition('alb_instance', $alb_instance)
    ->execute()
    ->fetchObject();
}
