<?php

/**
 * The claim collection.
 */
class AlbAuthClaimCollection implements AlbAuthClaimCollectionInterface {

  /**
   * An array of claims.
   *
   * @var AlbAuthClaimInterface[]
   */
  protected $claims;

  /**
   * Constructs a ClaimCollection object.
   *
   * @param AlbAuthClaimInterface[] $claims
   *   An array of claims.
   */
  public function __construct(array $claims) {
    $this->claims = [];
    foreach ($claims as $claim) {
      if (!$claim instanceof AlbAuthClaimInterface) {
        throw new \InvalidArgumentException();
      }
      $this->claims[$claim->getName()] = $claim;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function fromArray(array $data) {
    $claims = [];
    foreach ($data as $name => $value) {
      $claims[] = new AlbAuthClaim($name, $value);
    }
    return new static($claims);
  }

  /**
   * {@inheritdoc}
   */
  public function getClaims() {
    return $this->claims;
  }

  /**
   * {@inheritdoc}
   */
  public function getClaimByName($name) {
    return isset($this->claims[$name]) ? $this->claims[$name] : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getClaimValue($name, $default = NULL) {
    $claim = $this->getClaimByName($name);
    return $claim ? $claim->getValue() : $default;
  }

  /**
   * {@inheritdoc}
   */
  public function hasData() {
    return !empty($this->claims);
  }

  /**
   * {@inheritdoc}
   */
  public function toArray() {
    $array = [];
    foreach ($this->getClaims() as $claim) {
      $array[$claim->getName()] = $claim->getValue();
    }
    return $array;
  }

  /**
   * {@inheritdoc}
   */
  public function hash() {
    $array = $this->toArray();

    $excluded_claims = [
      'exp',
      'nbf',
      'iat',
      'jti',
    ];
    // The AWS returns claims with the values which can be changed every on
    // request. This causes the module to update the user on each request. So,
    // we remove these claims to generate a proper claims hash.
    // @see alb_auth_update_user_with_claims();
    $array = array_diff_key($array, array_flip($excluded_claims));
    asort($array);
    return sha1(serialize($array));
  }

}
